// File: Payroll.cpp
// Creates a company employee payroll file
//    computes total company payroll amount

#include <fstream>   // required for file streams
#include <cstdlib>   // for definition of EXIT_FAILURE
#include "money.h"
using namespace std;

// Associate streams with external file names
#define inFile "EmpFile.txt"  // employee file
#define outFile "Salary.txt"  // payroll file

// Functions used ...
// PROCESS ALL EMPLOYEES AND COMPUTE TOTAL
money processEmp(istream&, ostream&);

int main()
{
   ifstream eds;        // input: employee data stream
   ofstream pds;        // output: payroll data stream
   money totalPayroll;  // output: total payroll

   // Prepare files.
   eds.open(inFile);
   if (eds.fail ())
   {
      cerr << "*** ERROR: Cannot open " << inFile
             << " for input." << endl;
      return EXIT_FAILURE;	// failure return
   }

   pds.open(outFile);
   if (pds.fail())
   {
      cerr << "***ERROR: Cannot open " << outFile
             << " for output." << endl;
      eds.close();
      return EXIT_FAILURE;  // failure return
   }

// Process all employees and compute total payroll.
   totalPayroll = processEmp(eds, pds);

   // Display result.
   cout << "Total payroll is " << totalPayroll << endl;

   // Close files.
   eds.close();
   pds.close();
   
   return 0;
}

// Insert processEmp here.
// Process all employees and compute total payroll amount
// Pre:  eds and pds are prepared for input/output.
// Post: Employee names and salaries are written from eds to pds
//       and the sum of their salaries is returned.
// Returns: Total company payroll
money processEmp 
  (istream& eds,              // IN: employee file stream
   ostream& pds)              // IN: payroll file stream
{
   string firstName;   // input: employee first name
   string lastName;    // input: employee last name
   float hours;        // input: hoursWorked
   money rate;         // input: hourly rate
   money salary;       // output: gross salary
   money payroll;      // return value - total company payroll

   payroll = 0.0;
   // Read first employee's data record.
   //eds >> firstName >> lastName >> hours >> rate;
   //getline(eds, firstName);
   //eds.getline(firstName, 100, '\n');
   getline(eds, firstName, '#');

   eds >> hours >> rate;
   while (!eds.eof())
   {
      salary = hours * rate;
      pds << firstName << " " << lastName
          << " " << salary << endl;
      payroll += salary;

      // Read next employee's data record.
      // eds >> firstName >> lastName >> hours >> rate;
      eds.ignore(100, '\n');
      getline(eds, firstName, '#');
      eds >> hours >> rate;
   }  // end while

   return payroll;
}  // end processEmp

