// File: arrayOperations.cpp
// Finds the subscript of the smallest value in a subarray.

// Returns the subscript of the smallest value in the subarray
// consisting of elements x[startindex] through x[endindex]
// Returns -1 if the subarray bounds are invalid.
// Pre:  The subarray is defined and 0 <= startIndex <= endIndex.
// Post: x[minIndex] is the smallest value in the array.

int findIndexOfMin
   (const float x[],       // IN: array of elements
    int startIndex,        // IN: subscript of first element
    int endIndex)          // IN: subscript of last element
{
   // Local data ...
   int minIndex;            // index of the smallest element found
   int i;                   // index of the current element

   // Validate subarray bounds
   if ((startIndex < 0) || (startIndex > endIndex))
   {
      cerr << "Error in subarray bounds" << endl;
      return -1;                        // return error indicator
   }

   // Assume the first element of subarray is smallest and check 
   //   the rest.
   // minIndex will contain subscript of smallest examined so far.
   minIndex = startIndex;
   for (i = startIndex + 1; i <= endIndex; i++)
      if (x[i] < x[minIndex])
         minIndex = i;

   // All elements are examined and minIndex is
   //    the index of the smallest element.
   return minIndex;                    // return result
} // end findIndexOfMin


// Searches an integer array for a given element (the target)

// Array elements ranging from 0 to size - 1 are searched for
//    an element equal to target.
// Pre:  The target and array are defined.
// Post: Returns the subscript of target if found;
//       otherwise, returns -1.

int linSearch
  (const int items[],          // IN: the array being searched
   int target,                 // IN: the target being sought
   int size)                   // IN: the size of the array
{
   for (int i = 0, i < size, i++)
      if (items[next] == target)
         return next;            // found, return subscript

   // All elements were tested without success.
   return -1;
}  // end linSearch



// Sorts an array (ascending order) using selection sort algorithm
//    Uses exchange and findIndexOfMin

// Sorts the data in array items (items[0] through items[n-1]).
// Pre:  items is defined and n <= declared size of actual argument
//       array.
// Post: The values in items[0] through items[n-1] are in 
//       increasing order.

void selSort(int items[], int n)
{
   // Local data ...
   int minSub;  // subscript of each smallest item located by 
                // findIndexOfMin

   for (int i = 0; i < n-1; i++)
   {
      // Find index of smallest element in unsorted section of 
      //    items.
      minSub = findIndexOfMin(items, i, n-1);

      // Exchange items at position minSub and i
      exchange(items[minSub], items[i]);
   }
} 

