// File: binaryTree.cpp
// Implementation of template class binary search tree

#include "binaryTree.h"
#include <iostream>
#include <cstdlib>
using namespace std;

   // Member functions ...
   // constructor - create an empty tree
   template<class treeElement>
   binaryTree<treeElement>::binaryTree()
   {
      root = NULL;
   }

  // Searches for the item with same key as el
  //  in a binary search tree.
  // Pre : el is defined.
  // Returns true if el's key is located,
  //   otherwise, returns false.
  template<class treeElement>
  bool binaryTree<treeElement>::search
    (const treeElement& el) const   // IN: Element to search for
   {
     return search(root, el); // Start search at tree root.
   } // search

   // Searches for the item with same key as el in the
   // subtree pointed to by aRoot. Called by public search.
   // Pre : el and aRoot are defined.
   // Returns true if el's key is located,
   //   otherwise, returns false.
   template<class treeElement>
   bool binaryTree<treeElement>::search
     (treeNode* aRoot,             // IN: Subtree to search
      const treeElement& el) const // IN: Element to search for
   {
    if (aRoot == NULL)
      return false;                      // Tree is empty.
    else if (el == aRoot->info)
      return true;                       // Target is found.
    else if (el <= aRoot->info)
      return search(aRoot->left, el);  // Search left.
    else
      return search(aRoot->right, el); // Search right.
  } // search

  // Inserts item el into a binary search tree.
// Pre : el is defined.
// Post: Inserts el if el is not in the tree.
//       Returns true if the insertion is performed.
//       If there is a node with the same key value as el,
//       returns false.
template<class treeElement>
bool binaryTree<treeElement>::insert
  (const treeElement& el)  // IN - item to insert
{
    return insert(root, el);
} // insert


// Inserts item el in the tree pointed to by aRoot.
// Called by public insert.
// Pre : aRoot and el are defined.
// Post: If a node with same key as el is found,
//       returns false. If an empty tree is reached,
//       inserts el as a leaf node and returns true.
template<class treeElement>
bool binaryTree<treeElement>::insert
  (treeNode*& aRoot,      // INOUT : Insertion subtree
   const treeElement& el) // IN    : Element to insert
{
  // Check for empty tree.
  if (aRoot == NULL)
  { // Attach new node
    aRoot = new treeNode;   // Connect aRoot to new node.
    aRoot->left = NULL;     // Make new node a leaf.
    aRoot->right = NULL;
    aRoot->info = el;       // Place el in new node
    return true;
  }
  else if (el == aRoot->info)
    return false;                // duplicate key found.
  else if (el <= aRoot->info)
    return insert(aRoot->left, el);  // insert left.
  else
    return insert(aRoot->right, el); // insert right.
} // insert

// Displays a binary search tree in key order.
// Pre : none
// Post: Each element of the tree is displayed.
//       Elements are displayed in key order.
template<class treeElement>
void binaryTree<treeElement>::display() const
{
   display(root);
} // display


// Displays the binary search tree pointed to
// by aRoot in key order. Called by display.
// Pre : aRoot is defined.
// Post: displays each node in key order.
template<class treeElement>
void binaryTree<treeElement>::display
  (treeNode* aRoot) const         // IN : Subtree to display
{
   if (aRoot != NULL)
   { // recursive step
     display(aRoot->left);           // display left subtree.
     cout << aRoot->info << endl;    // display root item.
     display(aRoot->right);          // display right subtree.
   }
} // display

// Insert member functions retrieve.
  template<class treeElement>
  bool binaryTree<treeElement>::retrieve
    (const treeElement& el) const   // IN: Element to search for
   {
     return retrieve(root, el); // Start search at tree root.
   } // retrieve
   // Retrieves for the item with same key as el in the
   // subtree pointed to by aRoot. Called by public search.
   // Pre : el and aRoot are defined.
   // Returns true if el's key is located,
   //   otherwise, returns false.
   template<class treeElement>
   bool binaryTree<treeElement>::retrieve
     (treeNode* aRoot,             // IN: Subtree to search
      treeElement& el) const // IN: Element to search for
      {
         return true;
      }   

