// File: binaryTree.h
// Definition of template class binary search tree


#ifndef BINARY_TREE_H
#define BINARY_TREE_H

// Specification of the class binTree<treeElement>
// Elements:      A tree consists of a collection of elements
//                that are all of the same type, treeElement.
// Structure:     Each node of a binary search tree has zero, one, 
//                or two subtrees connected to it. The key value in 
//                each node of a binary search tree is larger than 
//                all key values in its left subtree and smaller 
//                than all key values in its right subtree.

template<class treeElement>
class binaryTree
{
   public:
      // Member functions ...
      // constructor - create an empty tree
      binaryTree();

      // Insert an element into the tree
      bool insert
        (const treeElement& el );  // IN: Element to insert

      // Retrieve an element from the tree
      bool retrieve
        (const treeElement& el) const;  // OUT: element retrieved

      // Search for an element in the tree
      bool search
        (const treeElement& el) const;  // IN: element being 
                                         //     searched for

      // Display a tree
      void display() const;

   private:
      // Data type ...
      struct treeNode
      {
         treeElement info;      // the node data
         treeNode* left;        // pointer to left-subtree
         treeNode* right;       // pointer to right-subtree
      };

      // Data member 
      treeNode* root;           // the root of the tree

      // Private member functions ...
      // Searches a subtree for a key
      bool search(treeNode*,              // root of a subtree
                  const treeElement&) const; // key being 
                                                // searched for
      // Inserts an item in a subtree
      bool insert(treeNode*&,           // root of a subtree
                  const treeElement&);  // item being inserted

      // Retrieves an item in a subtree
      bool retrieve(treeNode*,            // root of a subtree
                    treeElement&) const; // item to retrieve

      // Displays a subtree
      void display(treeNode*) const;

};

#endif  // BINARY_TREE_H


