// File: counter.cpp
// Counter class implementation

#include "counter.h"

#include <iostream>
#include <limits>          // For INT_MAX
using namespace std;

// Default constructor
counter::counter()
{
   count = 0;
   maxValue = INT_MAX;    // Set maxValue to default maximum
}  


// Constructor with argument
counter::counter(int mVal)   // IN: maximum integer value
{
   count = 0;
   maxValue = mVal;      // Set maxValue to mVal 
}  


// Increment counter
void counter::increment()
{
   if (count < maxValue)
      count++;
   else
      cerr << "Counter overflow. Increment ignored." << endl;
}  


// Decrement counter
void counter::decrement()
{
   if (count > 0)
      count--;
   else
      cerr << "Counter underflow. Decrement ignored." << endl;
}  


// Set counter value
void counter::setCount(int val)
{
   if (val >= 0  &&  val <= maxValue)
      count = val;
   else
      cerr << "New value is out of range. Value not changed." 
           << endl;
}


// Set maximum counter value
void counter::setMaxValue(int val)
{
   if (val >= 0  &&  val <= INT_MAX)
      maxValue = val;
   else
      cerr << "New maxValue is out of range -- not changed." 
           << endl;
}


// Return current counter value
int counter::getCount() const
{
   return count;
}  


// Return maximum counter value
int counter::getMaxValue() const
{
   return maxValue;
}  

