// File: indexList.cpp
// Indexed list class implementation

#include "indexList.h"
#include <iostream>
using namespace std;

template <class T, int maxSize>
indexList<T, maxSize>::indexList()    // constructor
{
	size = 0;		// list is empty
}

// Add an item to the end of an indexed list
template <class T, int maxSize>
bool indexList<T, maxSize>::append(const T& item)

// Pre:  item is defined
// Post: if size < maxSize, item is appended to list
// Returns: true if item was appended; otherwise, false
{
	bool result;

	// Add item to the end of the list if list is not full.
	if (size < maxSize)
	{
		elements[size] = item;
		size++;
		result = true;
	}
	else
	{
		cerr << "Array is filled - can't append!" << endl;
		result = false;
	}
	return result;
}

// Replace an item at a specified index with a new one.
// Pre:  item and index are defined
// Post: item overwrites old item at position index if valid
// Returns: true if item was inserted; otherwise, false

template <class T, int maxSize>
bool indexList<T, maxSize>::replace(int index, const T& item)
{
	bool result;

	// Overwrite a list element if index is valid.
	if (index >= 0  &&  index < size)
	{
		elements[index] = item;
		result = true;
	}
	else
	{
		cerr << "Index " << index << " not in filled part"
			<< " - can't insert!" << endl;
		result = false;
	}
	return result;
}

template <class T, int maxSize>
bool indexList<T, maxSize>::insert(int index, const T& item)
{
	bool result;
        int i;

	// Insert a list element if index is valid.
	if (index >= 0  &&  index < size && size < maxSize)
	{
                // Shift elements down 1 position
                for (i = size - 1; i >= index; i--)
                    elements[i + 1] = elements[i];
		elements[index] = item;  // insert
                size++;
		result = true;
	}
	else
	{
		cerr << "Index " << index << " not in filled part"
			<< " - can't insert!" << endl;
		result = false;
	}
	return result;
}

// Retrieve an item at a specified index
// Pre:  item and index are defined
// Post: if index is valid, elements[index] is returned
// Returns: true if item was returned; otherwise, false

template <class T, int maxSize>
bool indexList<T, maxSize>::retrieve(int index, T& item) const
{
	bool result;

	// Return a list element through item if index is valid.
	if (index >= 0  &&  index < size)
	{
		item = elements[index];
		result = true;
	}
	else
	{
		cerr << "Index " << index << " not in filled part"
		     << " - can't retrieve!" << endl;
		result = false;
	}
	return result;
}

// Delete an element at a specified index
// Pre:  index is defined
// Post: if index is valid, elements[index] is replaced
//       with elements[size] and size is decremented.
// Returns: true if item was deleted; otherwise, false

template <class T, int maxSize>
bool indexList<T, maxSize>::remove(int index)

{
	bool result;
        int i;

	// Delete element at index i by moving elements up
	if (index >= 0  &&  index < size)
	{
		// Shift each element up 1 position
		for (i = index + 1; i < size; i++)
                   elements[i-1] = elements[i];
		size--;           // Decrement size
		result = true;
	}
	else
	{
		cerr << "Index " << index << " not in filled part"
		     << " - can't delete!" << endl;
		result = false;
	}
	return result;
}
/*
// Read data into the list
// Pre:  none
// Post: All data items are stored in array elements
//       and size is the count of items

template <class T, int maxSize>
void indexList<T, maxSize>::read(istream& ins)
{
	T nextItem;		// input - next data item

	size = 0;                // The list is empty.
        ins >> nextItem;
	while (!ins.eof() && size < maxSize)
        {
	   elements[size] = nextItem;
           size++;
           cout << "Enter next item - ";
           ins >> nextItem;
        }
}
*/

// Read data into the list
// Pre:  none
// Post: All data items are stored in array elements
//       and size is the count of items

template <class T, int maxSize>
void indexList<T, maxSize>::read(istream& ins)
{
	int numItems;	// input - number of items to read
	T nextItem;		// input - next data item

	cout << "Enter number of list items to read: ";
	ins >> numItems;
	ins.ignore(80, '\n');   // skip newline

	// If numItems is valid, read each list element,
	// starting with first element.
	size = 0;                // The list is empty.
	if (numItems >= 0  &&  numItems <= maxSize)
	   while (size < numItems)
	   {
	      cout << "Enter next item - ";
	      ins >> nextItem;
	      elements[size] = nextItem;
              size++;
	   }
	else
	   cerr << "Number of items " << numItems
           << " is invalid"
	        << " - data entry is cancelled!" << endl;
}

// Display the list contents
// Pre:  none
// Post: Displays each item stored in the list

template <class T, int maxSize>
void indexList<T, maxSize>::display() const
{
	// Display each list element.
	for (int i = 0; i < size; i++)
		cout << elements[i] << endl;
}


// Find index of a target item
// Pre:  none
// Post: Returns the index of target if found;
//         otherwise, return -1.

template <class T, int maxSize>
int indexList<T, maxSize>::search(const T& target) const
{
	for (int i = 0; i < maxSize; i++)
		if (elements[i] == target)
			return i;       // target found at position i

	// target not found
	return -1;
}


// Get the current size
template <class T, int maxSize>
int indexList<T, maxSize>::getSize() const
{
	return size;
}


// Sort the indexed list
template <class T, int maxSize>
void indexList<T, maxSize>::selSort()
{
	// Selection sort stub - do nothing
}


