// File: indexList.h
// Definition of indexed list template class

#ifndef INDEXLIST_H
#define INDEXLIST_H

#include <iostream>
using namespace std;

template <class T, int maxSize>
class indexList
{
public:
	// Constructor
	indexList();

	// Add an item to the end of an indexed list
	bool append(const T&);   // IN: item appended

	// Overwrite an element at a specified index
	bool replace(int,         // IN: insertion index
	             const T&);   // IN: item inserted

	// Insert an element at a specified index
	bool insert(int,         // IN: insertion index
	            const T&);   // IN: item inserted

        // Retrieve an element at a specified index
	bool retrieve(int,       // IN:  index
	              T&) const; // OUT: value retrieved

	// Delete an element at a specified index
	bool remove(int);        // IN: index

	// Find index of smallest value in a sublist
	int findMin(int,        // IN:  start index
	            int) const; // OUT: end index

	// Find index of largest value in a sublist
	int findMax(int,        // IN:  start index
 	            int) const; // OUT: end index

	// Find index of a target item
	//    Returns -1 if target item not found
	int search(const T&) const;  // IN: target item

	// Sort an indexed list
	void selSort();

	// Read data into the list
        void read(istream&);

	// Display the list contents
	void display() const;

	// Get the current size
	int getSize() const;

private:
	T elements[maxSize];  // Storage for elements
	int size;             // Count of elements in list
};

#endif  // INDEXLIST_H
