// File: payrollFunctions.cpp
// Computes and displays gross pay and net pay given an hourly
//    rate and number of hours worked.  Deducts union dues of $15
//    if gross salary exceeds $100; otherwise, deducts no dues.

#include <iostream>
#include "money.h"
using namespace std;

// Functions used ...
void instructUser();

money computeGross(float, money);

money computeNet(money);

const money MAX_NO_DUES = 100.00;  // max earnings before dues (dollars)
const money dues = 15.00;          // dues amount (dollars)
const float MAX_NO_OVERTIME = 40.0;// max hours before overtime 
const float OVERTIME_RATE = 1.5;   // overtime rate

int main ()
{
   float hours;                // input: hours worked
   float rate;                 // input: hourly pay rate (dollars)
   money gross;                // output: gross pay (dollars)
   money net;                  // output: net pay (dollars)

   // Display user instructions.
   instructUser();

   // Enter hours and rate.
   cout << "Hours worked: ";
   cin  >> hours;
   cout << "Hourly rate: ";
   cin  >> rate;

   // Compute gross salary.
   gross = computeGross(hours, rate);

   // Compute net salary.
   net = computeNet(gross);

   // Print gross and net.
   cout << "Gross salary is " << gross << endl;
   cout << "Net salary is " << net << endl;

   getch();
   return 0;
}

// Insert lower-level functions here.
// ...
// Displays user instructions
void instructUser()
{
   cout << "This program computes gross and net salary." << endl;
   cout << "A dues amount of " << dues << " is deducted for" << endl;
   cout << "an employee who earns more than " << MAX_NO_DUES 
        << endl << endl;
   cout << "Overtime is paid at the rate of " << OVERTIME_RATE << endl;
   cout << "times the regular rate for hours worked over " 
        << MAX_NO_OVERTIME
        << endl << endl;
   cout << "Enter hours worked and hourly rate" << endl;
   cout << "on separate lines after the prompts." << endl;
   cout << "Press <return> after typing each number." << endl << endl;
}  // end instructUser


// FIND THE GROSS PAY
money computeGross
  (float hours,                 // IN: number of hours worked
   money rate)                  // IN: hourly pay rate (dollars)
{
   // Local data ...
   money gross;                 // RESULT: gross pay (dollars) 
   money regularPay;           // pay for first 40 hours
   money overtimePay;          // pay for hours in excess of 40

   // Compute gross pay.
   if (hours > MAX_NO_OVERTIME)
   {
      regularPay = MAX_NO_OVERTIME * rate;
      overtimePay = (hours - MAX_NO_OVERTIME) * OVERTIME_RATE * rate;
      gross = regularPay + overtimePay;
   }
   else 
      gross = hours * rate;

   return gross;
}  // end computeGross

// Find the net pay
money computeNet
  (money gross) 			// IN: gross salary (dollars)
{
   // Local data ...
   money net;                  // RESULT: net pay (dollars) 

   // Compute net pay.
   if (gross > MAX_NO_DUES)
      net = gross - dues;  	// deduct dues amount
   else
      net = gross;  	// no deductions

   return net;
}  // end computeNet

