// File: queue.h
// Definition of a template class queue using a linked list

#ifndef QUEUE_H
#define QUEUE_H

// Specification of the class queue<queueElement>
// Elements:      A queue consists of a collection of elements
//                that are all of the same type, queueElement.
// Structure:     The elements of a queue are ordered according
//                to time of arrival. The element that was first
//                inserted into the queue is the only one that
//                may be removed or examined. Elements are
//                removed from the front of the queue and
//                inserted at the rear of the queue.

template<class queueElement>
class queue
{
   public:
      // Member functions ...
      // constructor - create an empty queue
      queue();

      // Insert an element into the queue
      bool insert
        (const queueElement& x);   // IN: Element to insert

      // Remove an element from the queue
      bool remove
        (queueElement& x);         // OUT: element removed

      // Test for empty queue
      bool isEmpty();

      // Get queue size
      int getSize();

   private:
      // Data members ...
      struct queueNode
      {
         queueElement item;
         queueNode* next;
      };

      queueNode* front;      // the front of the queue
      queueNode* rear;       // the back of the queue
      int numItems;          // the number of items currently
                             //    in the queue
};

#endif   // QUEUE_H

