// File: stack.cpp
// Implementation of template class stack

#include "stack.h"

// constructor to create an empty stack

template <class stackElType, int maxSize>
stack<stackElType, maxSize>::stack ()
{
   top = -1;            // A value of -1 indicates empty
} 


// Push an element onto the stack
// Pre:  The element x is defined.
// Post: If the stack is not full, the item is pushed onto
//       the stack and true is returned. Otherwise, the stack
//       is unchanged and false is returned.

template <class stackElType, int maxSize>
bool stack<stackElType, maxSize>::push
   (const stackElType& x)    // IN: item to push onto stack
{
   bool success;        // flag: true indicates successful push

   if (top < maxSize - 1)     // If there is room
   {
      top++;                   // increment top
      items[top] = x;          // insert x
      success = true;          // indicate success
   }
   else
      success = false;   // no room, indicate failure

   return success;
}  // end push


// Pop an element off the stack
// Pre:  none
// Post: If the stack is not empty, the value at the top of
//       the stack is removed, its value is placed in x, and
//       true is returned. If the stack is empty, x is not
//       defined and false is returned.

template <class stackElType, int maxSize>
bool stack<stackElType, maxSize>::pop
   (stackElType& x)   // OUT: Element popped from stack
{
   bool success;        // flag: true indicates successful pop
 
   if (top >= 0)             	// if not empty
   {
      x = items[top];      	// remove top element
      top--;                 	// decrement top
      success = true;       	// indicate success
   }
   else
      success = false;    	// indicate failure
 
   return success;
}  // end pop
 
 
// Access top element from stack without popping
// Pre:  none
// Post: If the stack is not empty, the value at the top is
//       copied into x and true is returned. If the stack is
//       empty, x is not defined and false is returned. In either
//       case, the stack is not changed.

template <class stackElType, int maxSize>
bool stack<stackElType, maxSize>::peek
   (stackElType& x)// OUT: Value at top of stack
   const
{
   bool success;        // flag: true indicates successful get
   if (top >= 0)                // if not empty
   {
      x = items[top];           // retrieve top element, do not pop
      success = true;           // indicate success
   }
   else
      success = false;          // indicate failure
 
   return success;
}  // end peek


// Test to see if stack is empty
// Pre:  none
// Post: Returns true if the stack is empty; otherwise,
//       returns false.

template <class stackElType, int maxSize>
bool stack<stackElType, maxSize>::isEmpty() const
{
   return top < 0;
}  


// Test to see if stack is full
// Pre:  none
// Post: Returns true if the stack is full; otherwise,
//       returns false.

template <class stackElType, int maxSize>
bool stack<stackElType, maxSize>::isFull() const

{
   return top >= maxSize-1;
} 
