// File: stack.h
// Definition for a template class stack

// Structure:     A stack consists of a collection of elements
//                that are all of the same type, stackElType.
//                The elements of a stack are ordered according
//                to when they were placed on the stack. Only
//                the element that was last inserted onto the
//                stack can be removed or examined. New elements
//                are inserted at the top of the stack. Space for
//                maxSize elements is allocated, default is 100.

#ifndef STACK_H
#define STACK_H

template <class stackElType, int maxSize = 100>
class stack
{
   public:
      // Member functions
      // constructor to create an empty stack
      stack();

      // Push an element onto the stack
      bool push
        (const stackElType& x);  // IN: item to push onto stack

      // Pop an element off the stack
      bool pop
        (stackElType& x);   // OUT: Element popped from stack

      // Retrieve top element from stack without popping
      bool peek
        (stackElType& x) const; // OUT: Value at top of stack

      // Test to see if stack is empty
      bool isEmpty() const;

      // Test to see if stack is full
      bool isFull() const;

   private:
      // The data members are explained in Section 11.9
      int top;             // index of element at top of stack
      stackElType items[maxSize];   // array storage for stack
};

#endif //STACK_H
