      // File: stackList.cpp
      // Implementation of template class stack as a linked list

      #include "stackList.h"
      #include <cstdlib>               // for NULL
      using namespace std;  // delete for Visual C++

      // Member functions ...
      // constructor to create an empty stack
      template <class stackElement>
      stackList<stackElement>::stackList()
      {
         top = NULL;
      }  // end stackList

      // Push an element onto the stack
      // Pre:  The element x is defined.
      // Post: If there is space on the heap, the item is pushed
      //       onto the stack and true is returned. Otherwise, the
      //       stack is unchanged and false is returned.
      template <class stackElement>
      bool stackList<stackElement>::push
        (const stackElement& x)  // IN: Element pushed onto stack
      {
         // Local data 
         stackNode* oldTop;
         bool success;           // program flag - indicates 
                                 //   success or failure

         oldTop = top;          // save old top
         top = new stackNode;   // allocate new node at top of stack
         if (top == NULL)      // check to see if new was successful
         {
            top = oldTop;       // if not, restore top
            success = false;    // indicate push failed
         }
         else
         {
            top->next = oldTop;  // link new node to old stack
            top->item = x;       // store x in new node
            success = true;      // indicate success
         }
         return success;
      }  // end push


      // Pop an element off the stack
      // Pre:  none
      // Post: If the stack is not empty, the value at the top
      //       of the stack is removed, its value is placed in
      //       x, and true is returned. If the stack is empty, x is
      //       not defined and false is returned.
      template <class stackElement>
      bool stackList<stackElement>::pop
        (stackElement& x)   // OUT: Element popped from stack
      {
         // Local data
         stackNode* oldTop;
         bool success;          // program flag - indicates 
                                //   success or failure

         if (top == NULL)
            success = false;
         else
         {
            x = top->item;         // copy top of stack into x
            oldTop = top;          // save old top of stack
            top = oldTop->next;    // reset top of stack
            delete oldTop;         // return top node to the heap
            success = true;        // indicate success
         }
         return success;
      }  // end pop


      // Get top element from stack without popping
      // Pre:  none
      // Post: If the stack is not empty, the value at the top is
      //       copied into x and true is returned. If the stack is
      //       empty, x is not defined and false is returned. In 
      //       either case, the stack is not changed.
      template <class stackElement>
      bool stackList<stackElement>::peek
        (stackElement& x) const  // OUT: Value returned from stack 
      {
         // Local data 
         bool success;            // program flag - indicates 
                                  //   success or failure

         if (top == NULL)
            success = false;
         else
         {
            x = top->item;
            success = true;
         }
         return success;
      }  // end peek


      // Test to see if stack is empty
      // Pre : none
      // Post: Returns true if the stack is empty; otherwise,
      //       returns false.
      template <class stackElement>
      bool stackList<stackElement>::isEmpty() const
      {
         return top == NULL;
      }  // end isEmpty


      // Test to see if stack is full
      // Pre : none
      // Post: Returns false. List stacks are never full. (Does not
      //       check heap availability.)
      template <class stackElement>
      bool stackList<stackElement>::isFull() const
      {
         return false;
      }  // end isFull
