// File: stackList.h
// Definition of a template class stackList using a linked list

#ifndef STACK_LIST_H
#define STACK_LIST_H

template <class stackElement>
class stackList
{
   public:
      // Member functions ...
      // constructor to create an empty stack
      stackList();

      // Push an element onto the stack
      // Pre : The element x is defined.
      // Post: If the stack is not full, the item is pushed onto 
      //       the stack and true is returned. Otherwise, the 
      //       stack is unchanged and false is returned.
      bool push
        (const stackElement& x); // IN: item pushed onto stack

      // Pop an element off the stack
      // Pre : none
      // Post: If the stack is not empty, the value at the top
      //       of the stack is removed, its value is placed in
      //       x, and true is returned. If the stack is empty, x 
      //       is not defined and false is returned.
      bool pop
        (stackElement& x);   // OUT: Element popped from stack

      // Get top element from stack without popping
      // Pre : none
      // Post: If the stack is not empty, the value at the top 
      //       is copied into x and true is returned. If the 
      //       stack is empty, x is not defined and false is 
      //       returned. In either case, the stack is not 
      //       changed.
      bool peek
        (stackElement& x) const; // OUT: Value returned 
                                  //   from top of stack

      // Test to see if stack is empty
      // Pre : none
      // Post: Returns true if the stack is empty; otherwise,
      //       returns false.
      bool isEmpty() const;

      // Test to see if stack is full
      // Pre : none
      // Post: Returns true if the stack is full; otherwise,
      //       returns false.
      bool isFull() const;

   private:
      struct stackNode
      {
         stackElement item;   	// storage for the node data
         stackNode* next;     	// link to next node
      };

      // Data member
      stackNode* top;      // pointer to node at top of stack
};

#endif  // STACK_LIST_H
