// FILE: TelDirecMenu.cpp
// MENU DRIVEN TELEPHONE DIRECTORY UPDATE PROGRAM

#include "indexList.h"
#include "entry.h"
#include <cctype>         // For toupper
#include <iostream>
#include <string>
using namespace std;

typedef indexList<entry, 100> telIndexList;

// Function prototype
   // PERFORMS USER SELECTION
   void select(telIndexList&, char);

int main()
{
   telIndexList telDirec;           // telephone directory
   char choice;                     // menu choice

   // Read the initial directory.
   cout << "Enter the initial directory entries -" << endl;
   telDirec.read(cin);

   // Keep reading and performing operations until user enters Q
   do
   {
      // Display the menu.
      cout << "Enter your choice -" << endl;
      cout << "A(Add), C(Change), D(Delete)," << endl 
           << "G(Get), S(Sort),   P(Print), Q(Quit): ";

      cin >> choice;
      cin.ignore(80, '\n');    // skip newline

      // Perform the operation selected.
      select(telDirec, choice);
   }
   while (toupper(choice) != 'Q');
   
   return 0;
}


// Performs user selection
void select(telIndexList& telDirec,  // INOUT : directory
            char choice)             // IN: selection
{
   // Local data
   entry anEntry;     // one entry
   string aName;      // input - entry name
   int index;         // index of name in directory
   char answer;       // input - indicates whether to add
                      //         entry for missing name

   switch (toupper(choice))
   {
      case 'A':   // Add an entry
         cout << "Enter entry to add - ";
         cin >> anEntry;
         telDirec.append(anEntry);
         break;

      case 'C':   // Change an entry
         cout << "Enter entry to change - ";
         cin >> anEntry;
         index = telDirec.search(anEntry);
         if (index >= 0)
            telDirec.replace(index, anEntry);
         else
         {
            cout << "Name not in directory. "
                 << "Do you wish to add it (Y or N): ";
            cin >> answer;
            if (toupper(answer) == 'Y')
               telDirec.append(anEntry);
         }
         break;

      case 'D':   // Delete an entry
         cout << "Enter name of entry to delete: ";
         //cin.ignore(1, '\n');
         getline(cin, aName, '\n');
         anEntry.setEntry(aName);
         index = telDirec.search(anEntry);
         if (index >= 0)
            telDirec.remove(index);
         else
            cout << "Entry not found - no deletion" << endl;
         break;

      case 'G':   // Get a number
         cout << "Enter name of entry to get: ";
         //cin.ignore(1, '\n');
         getline(cin, aName, '\n');
         anEntry.setEntry(aName);
         index = telDirec.search(anEntry);
         if (index >= 0)
         {
            telDirec.retrieve(index, anEntry);
            cout << "The number you requested is "
                 << anEntry.getNumber() << endl;
         }
         else
            cout << "Not in directory." << endl;
         break;

      case 'S':   // Sort directory
         telDirec.selSort();
         break;

      case 'P':   // Print directory
         telDirec.display();
         break;

      case 'Q':   // Quit directory
         cout << "Exiting program" << endl;
         break;

      default:
         cout << "Choice is invalid - try again" << endl;
         //cin.ignore(80, '\n');
   }
}  // end select

